# Copyright (c) 2012: Thiago Chaves de Oliveira Horta <shundread@gmail.com>
#
# Authors: Thiago Chaves de Oliveira Horta
#
# License: GPL v3 (see gpl-3.0.txt)

from pygame.rect import Rect

from constant import *

class Entity(object):
    def __init__(self, sprite, position, solid):
        self.sprite = sprite
        self.position = position
        self.solid = solid
        self.state = 0

    # Drawing
    def draw(self, surface, shift=(0, 0)):
        self.sprite.draw(surface, self.get_position(), shift)

    # Positioning
    def get_position(self):
        return self.position

    def set_position(self, position):
        self.position = position

    def move(self, (dx, dy)):
        if dx or dy:
            if dx > 0:
                direction = Right
            elif dx < 0:
                direction = Left
            elif dy > 0:
                direction = Down
            elif dy < 0:
                direction = Up
            self.set_state(Move | direction)
        else:
            self.set_state(Move | self.get_direction())
        self.position = (self.position[0] + dx, self.position[1]+dy)

    def get_size(self):
        return self.sprite.get_size()

    def get_rect(self):
        return Rect(self.get_position(), self.get_size())

    # Solidity
    def is_solid(self):
        return self.solid

    # State setting and getting
    def set_direction(self, direction):
        self.set_state(self.get_action() | direction)

    def set_action(self, action):
        self.set_state(action | self.get_direction())

    def set_state(self, state):
        if state != self.state:
            self.state = state
            self.sprite.set_animation(self.state)

    def get_direction():
        return self.state & DirectionMask

    def get_action():
        return self.state & ActionMask

    def get_state():
        return self.state

    # Handling updates
    def update(self, game_data, dt):
        self.sprite.update(dt)
        
